@@ -9,24 +9,24 @@
 fsk_lib;
 
 
+% Basic modem set up for Horus
+
 function states = fsk_horus_init(Fs,Rs,M=2)
 
   states = fsk_init(Fs,Rs,M);
-  states.rtty = fsk_horus_init_rtty_uw(states);
-  states.binary = fsk_horus_init_binary_uw;
 
   % Freq. estimator limits - keep these narrow to stop errors with low SNR 4FSK
 
-  states.fest_fmin = 800;
-  states.fest_fmax = 2500;
-  states.fest_min_spacing = 200;
+  states.fest_fmin = 300;
+  states.fest_fmax = 2200;
+  states.fest_min_spacing = 100;
 
 endfunction
 
 
-% init rtty protocol specifc states
+% init rtty protocol specific states
 
-function rtty = fsk_horus_init_rtty_uw(states)
+function rtty = fsk_horus_init_rtty
   % Generate unque word that correlates against the ASCII "$$$$$" that
   % is at the start of each frame.
   % $ -> 36 decimal -> 0 1 0 0 1 0 0 binary 
@@ -35,11 +35,10 @@ function rtty = fsk_horus_init_rtty_uw(states)
   mapped_db = 2*dollar_bits - 1;
   sync_bits = [1 1 0];
   mapped_sb = 2*sync_bits - 1;
-  %mapped_sb = [ 0 0 0 ];
 
   mapped = [mapped_db mapped_sb];
-  npad = rtty.npad = 3;     % one start and two stop bits between 7 bit ascii chars
-  nfield = rtty.nfield = 7; % length of ascii character field
+  npad   = rtty.npad   = 3;     % one start and two stop bits between 7 bit ascii chars
+  nfield = rtty.nfield = 7;     % length of ascii character field
 
   rtty.uw = [mapped mapped mapped mapped mapped];
   rtty.uw_thresh = length(rtty.uw) - 2; % allow a few bit errors when looking for UW
@@ -49,7 +48,7 @@ endfunction
 
 % I think this is the binary protocol work from Jan 2016
 
-function binary = fsk_horus_init_binary_uw
+function binary = fsk_horus_init_binary
   % Generate 16 bit "$$" unique word that is at the front of every horus binary
   % packet
 
@@ -95,7 +94,7 @@ function [str crc_ok] = extract_ascii(states, rx_bits_buf, uw_loc)
   nfield = states.nfield;
   npad = states.npad;
 
-  str = []; str_dec = []; nstr = 0; ptx_crc = 1; rx_crc = "";
+  str = ""; str_dec = []; nstr = 0; ptx_crc = 1; rx_crc = "";
   endpacket = 0;
 
   st = uw_loc + length(states.uw);  % first bit of first char
@@ -185,20 +184,24 @@ endfunction
 
 % Extract as many ASCII packets as we can from a great big buffer of bits
 
-function extract_and_print_rtty_packets(states, rx_bits_log, rx_bits_sd_log)
+function npackets = extract_and_print_rtty_packets(states, rtty, rx_bits_log, rx_bits_sd_log)
 
   % use UWs to delimit start and end of data packets
 
   bit = 1;
   nbits = length(rx_bits_log);
-  nfield = states.rtty.nfield;
-  npad = states.rtty.npad;
-
-  uw_loc = find_uw(states.rtty, bit, rx_bits_log, states.verbose);
+  nfield = rtty.nfield;
+  npad = rtty.npad;
+  npackets = 0;
+  
+  uw_loc = find_uw(rtty, bit, rx_bits_log, states.verbose);
   
   while (uw_loc != -1)
-
-    if (uw_loc + states.rtty.max_packet_len) < nbits
+    if bitand(states.verbose,0x8)
+      printf("nbits: %d max_packet_len: %d uw_loc: %d\n", nbits, rtty.max_packet_len, uw_loc);
+    end
+    
+    if (uw_loc + rtty.max_packet_len) < nbits
       % Now start picking out 7 bit ascii chars from frame.  It has some
       % structure so we can guess where fields are.  I hope we don't get
       % RS232 idle bits stuck into it anywhere, ie "bit fields" don't
@@ -212,19 +215,20 @@ function extract_and_print_rtty_packets(states, rx_bits_log, rx_bits_sd_log)
       %rx_bits_log(st+200) = xor(rx_bits_log(st+100),1);
       %rx_bits_sd_log(st+100) = 0;
       
-      [str crc_ok] = extract_ascii(states.rtty, rx_bits_log, uw_loc);
+      [str crc_ok] = extract_ascii(rtty, rx_bits_log, uw_loc);
 
       if crc_ok == 0
-        [str_flipped crc_flipped_ok rx_bits_log] = sd_bit_flipping(states.rtty, rx_bits_log, rx_bits_sd_log, uw_loc, uw_loc+states.rtty.max_packet_len); 
+        [str_flipped crc_flipped_ok rx_bits_log] = sd_bit_flipping(rtty, rx_bits_log, rx_bits_sd_log, uw_loc, uw_loc+rtty.max_packet_len); 
       end
 
       % update memory of previous packet, we use this to guess where errors may be
       if crc_ok || crc_flipped_ok
-        states.prev_pkt = rx_bits_log(uw_loc+length(states.rtty.uw):uw_loc+states.rtty.max_packet_len);
+        states.prev_pkt = rx_bits_log(uw_loc+length(rtty.uw):uw_loc+rtty.max_packet_len);
       end
 
       if crc_ok
         str = sprintf("%s CRC OK", str);
+        npackets++;
       else
         if crc_flipped_ok
           str = sprintf("%s fixed", str_flipped);
@@ -237,8 +241,8 @@ function extract_and_print_rtty_packets(states, rx_bits_log, rx_bits_sd_log)
 
     % look for next packet
 
-    bit = uw_loc + length(states.rtty.uw);
-    uw_loc = find_uw(states.rtty, bit, rx_bits_log, states.verbose);
+    bit = uw_loc + length(rtty.uw);
+    uw_loc = find_uw(rtty, bit, rx_bits_log, states.verbose);
 
   endwhile
 endfunction
@@ -250,7 +254,7 @@ endfunction
 % compile with:
 %   codec2-dev/src$ gcc horus_l2.c -o horus_l2 -Wall -DDEC_RX_BITS -DHORUS_L2_RX
 
-function corr_log = extract_and_decode_binary_packets(states, rx_bits_log)
+function corr_log = extract_and_decode_binary_packets(states, binary, rx_bits_log)
   corr_log = [];
 
   % use UWs to delimit start and end of data packets
@@ -258,12 +262,12 @@ function corr_log = extract_and_decode_binary_packets(states, rx_bits_log)
   bit = 1;
   nbits = length(rx_bits_log);
 
-  [uw_loc best_corr corr] = find_uw(states.binary, bit, rx_bits_log, states.verbose);
+  [uw_loc best_corr corr] = find_uw(binary, bit, rx_bits_log, states.verbose);
   corr_log = [corr_log corr];
-
+  
   while (uw_loc != -1)
 
-    if (uw_loc+states.binary.max_packet_len) < nbits
+    if (uw_loc+binary.max_packet_len) < nbits
       % printf("uw_loc: %d best_corr: %d\n", uw_loc, best_corr);
 
       % OK we have a packet delimited by two UWs.  Lets convert the bit
@@ -290,11 +294,12 @@ function corr_log = extract_and_decode_binary_packets(states, rx_bits_log)
       system("../src/horus_l2");  % compile instructions above
     end
 
-    bit = uw_loc + length(states.binary.uw);
-    [uw_loc best_corr corr] = find_uw(states.binary, bit, rx_bits_log, states.verbose);
+    bit = uw_loc + length(binary.uw);
+    [uw_loc best_corr corr] = find_uw(binary, bit, rx_bits_log, states.verbose);
     corr_log = [corr_log corr];
    
   endwhile
+  
 endfunction
  
 
@@ -310,7 +315,7 @@ endfunction
 %                     which is the same as Fs=921600 Rs=115200
 %                     Uses packet based BER counter
 
-function run_sim(test_frame_mode, M=2, frames = 10, EbNodB = 100)
+function run_sim(test_frame_mode, M=2, frames = 10, EbNodB = 100, filename="fsk_horus.raw")
   timing_offset = 0.0; % see resample() for clock offset below
   fading = 0;          % modulates tx power at 2Hz with 20dB fade depth, 
                        % to simulate balloon rotating at end of mission
@@ -336,14 +341,18 @@ function run_sim(test_frame_mode, M=2, frames = 10, EbNodB = 100)
 
   if test_frame_mode == 4
     % horus rtty config ---------------------
-    states = fsk_horus_init(8000, 100);
-    states.tx_bits_file = "horus_tx_bits_rtty.txt"; % Octave file of bits we FSK modulate
+    states = fsk_horus_init(8000, 100, 2);
+    states.tx_bits_file = "horus_payload_rtty.txt"; % Octave file of bits we FSK modulate
+    rtty = fsk_horus_init_rtty;
+    states.ntestframebits = rtty.max_packet_len;
   end
                                
   if test_frame_mode == 5
     % horus binary config ---------------------
-    states = fsk_horus_init(8000, 50, 4);
+    states = fsk_horus_init(8000, 100, 4);
+    binary = fsk_horus_init_binary;
     states.tx_bits_file = "horus_tx_bits_binary.txt"; % Octave file of bits we FSK modulate
+    states.ntestframebits = binary.max_packet_len;
   end
 
   if test_frame_mode == 6
@@ -427,10 +436,25 @@ function run_sim(test_frame_mode, M=2, frames = 10, EbNodB = 100)
     test_frame = load(states.tx_bits_file);
     ltf = length(test_frame);
     ntest_frames = ceil((frames+1)*nbit/ltf);
-    tx_bits = [];
+    printf("Generating %d test packets\n", ntest_frames);
+
+    % 1 second of random bits to let estimators lock on
+
+    preamble = round(rand(1,states.Rs));
+
+    tx_bits = preamble;    
     for i=1:ntest_frames
       tx_bits = [tx_bits test_frame];
     end
+
+    % a packet len of random bits at end fill buffers to deocode final packet
+
+    if test_frame_mode == 4
+      postamble = round(rand(1,rtty.max_packet_len));
+    else
+      postamble = round(rand(1,binary.max_packet_len));
+    end
+    tx_bits = [tx_bits postamble];
   end
 
   if test_frame_mode == 6
@@ -461,7 +485,7 @@ function run_sim(test_frame_mode, M=2, frames = 10, EbNodB = 100)
 
   % dump simulated rx file
 
-  ftx=fopen("fsk_horus.raw","wb"); rxg = rx*1000; fwrite(ftx, rxg, "short"); fclose(ftx);
+  ftx=fopen(filename,"wb"); rxg = rx*1000; fwrite(ftx, rxg, "short"); fclose(ftx);
 
   timing_offset_samples = round(timing_offset*states.Ts);
   st = 1 + timing_offset_samples;
@@ -492,7 +516,7 @@ function run_sim(test_frame_mode, M=2, frames = 10, EbNodB = 100)
       % demodulate to stream of bits
 
       states = est_freq(states, sf, states.M);
-      states.f = 900 + 2*states.Rs*(1:states.M);
+      %states.f = 900 + 2*states.Rs*(1:states.M);
       %states.f = [1200 1400 1600 1800];
       [rx_bits states] = fsk_demod(states, sf);
 
@@ -527,11 +551,12 @@ function run_sim(test_frame_mode, M=2, frames = 10, EbNodB = 100)
   end
 
   if test_frame_mode == 4
-    extract_and_print_rtty_packets(states, rx_bits_log, rx_bits_sd_log)
+    npackets = extract_and_print_rtty_packets(states, rtty, rx_bits_log, rx_bits_sd_log);
+    printf("Received %d packets\n", npackets);
   end
 
   if test_frame_mode == 5
-    extract_and_decode_binary_packets(states, rx_bits_log);
+    extract_and_decode_binary_packets(states, binary, rx_bits_log);
   end
 
   figure(1);
@@ -588,24 +613,25 @@ function run_sim(test_frame_mode, M=2, frames = 10, EbNodB = 100)
 
 % demodulate a file of 8kHz 16bit short samples --------------------------------
 
-function rx_bits_log = demod_file(filename, test_frame_mode, noplot=0, EbNodB=100)
+function rx_bits_log = demod_file(filename, test_frame_mode=4, noplot=0, EbNodB=100)
   fin = fopen(filename,"rb"); 
   more off;
-
+  read_complex = 0;
+  
   %states = fsk_horus_init(96000, 1200);
 
   if test_frame_mode == 4
     % horus rtty config ---------------------
     states = fsk_horus_init(8000, 100, 2);
-    uwstates = fsk_horus_init_rtty_uw(states);
-    states.ntestframebits = states.nbits;
+    rtty = fsk_horus_init_rtty;
+    states.ntestframebits = rtty.max_packet_len;
   end
                                
   if test_frame_mode == 5
     % horus binary config ---------------------
-    states = fsk_horus_init(8000, 50, 4);
-    uwstates = fsk_horus_init_binary_uw;
-    states.ntestframebits = states.nbits;
+    states = fsk_horus_init(8000, 100, 4);
+    binary = fsk_horus_init_binary;
+    states.ntestframebits = binary.max_packet_len;
   end
 
   states.verbose = 0x1 + 0x8;
@@ -630,6 +656,20 @@ function rx_bits_log = demod_file(filename, test_frame_mode, noplot=0, EbNodB=10
     printf("length test frame: %d\n", states.ntestframebits);
   end
 
+  if test_frame_mode == 8
+    % test FSK modem using COTS tx --------------
+    states = fsk_init_hbr(96000, 8, 1200, 4, 16);
+    states.fest_fmin = 1000;
+    states.fest_fmax = 20000;
+    states.fest_min_spacing = 1000;
+    states.tx_bits_file = "../build_linux/src/tx_bit.bin";
+    states.verbose += 0x4;
+    ftmp = fopen(states.tx_bits_file, "rb"); test_frame = fread(ftmp,Inf,"char")'; fclose(ftmp);
+    states.ntestframebits = length(test_frame);
+    printf("length test frame: %d\n", states.ntestframebits);
+    read_complex = 1;
+  end
+
   N = states.N;
   P = states.P;
   Rs = states.Rs;
@@ -650,7 +690,11 @@ function rx_bits_log = demod_file(filename, test_frame_mode, noplot=0, EbNodB=10
   % optional noise.  Useful for testing performance of waveforms from real world modulators
 
   EbNo = 10^(EbNodB/10);
-  ftmp = fopen(filename,"rb"); s = fread(ftmp,Inf,"short"); fclose(ftmp); tx_pwr = var(s);
+  ftmp = fopen(filename,"rb"); s = fread(ftmp,Inf,"short");
+  if read_complex
+    s = s(1:2:end) + j*s(2:2:end);
+  end
+  fclose(ftmp); tx_pwr = var(s);
   variance = (tx_pwr/2)*states.Fs/(states.Rs*EbNo*states.bitspersymbol);
 
   % First extract raw bits from samples ------------------------------------------------------
@@ -663,7 +707,13 @@ function rx_bits_log = demod_file(filename, test_frame_mode, noplot=0, EbNodB=10
     % extract nin samples from input stream
 
     nin = states.nin;
-    [sf count] = fread(fin, nin, "short");
+    if read_complex
+      [sf count] = fread(fin, 2*nin, "short");
+      sf = sf(1:2:end) + j*sf(2:2:end);
+      count /= 2;
+    else
+      [sf count] = fread(fin, nin, "short");
+    end
     rx = [rx; sf];
     
     % add optional noise
@@ -680,7 +730,7 @@ function rx_bits_log = demod_file(filename, test_frame_mode, noplot=0, EbNodB=10
 
       states = est_freq(states, sf, states.M);
       %states.f = [1450 1590 1710 1850];
-      [rx_bits states] = fsk_horus_demod(states, sf);
+      [rx_bits states] = fsk_demod(states, sf);
 
       rx_bits_buf(1:states.ntestframebits) = rx_bits_buf(nbit+1:states.ntestframebits+nbit);
       rx_bits_buf(states.ntestframebits+1:states.ntestframebits+nbit) = rx_bits;
@@ -693,19 +743,20 @@ function rx_bits_log = demod_file(filename, test_frame_mode, noplot=0, EbNodB=10
       ppm_log = [ppm_log states.ppm];
       f_log = [f_log; states.f];
 
-      if test_frame_mode == 1
+      if (test_frame_mode == 1)
         states = ber_counter(states, test_frame, rx_bits_buf);
         if states.ber_state == 1
           states.verbose = 0;
         end
       end
-      if test_frame_mode == 6
+      if (test_frame_mode == 6)  || (test_frame_mode == 8)
         states = ber_counter_packet(states, test_frame, rx_bits_buf);
       end
      else      
       finished = 1;
     end
   end
+  printf("frames: %d\n", frames);
   fclose(fin);
 
   if noplot == 0
@@ -713,10 +764,11 @@ function rx_bits_log = demod_file(filename, test_frame_mode, noplot=0, EbNodB=10
 
     figure(1);
     plot(f_log);
-    hold off;
-
+    title('Tone Freq Estimates');
+    
     figure(2);
     plot(f_int_resample_log','+')
+    title('Integrator outputs for each tone');
 
     figure(3)
     clf
@@ -724,7 +776,6 @@ function rx_bits_log = demod_file(filename, test_frame_mode, noplot=0, EbNodB=10
     plot(norm_rx_timing_log)
     axis([1 frames -0.5 0.5])
     title('norm fine timing')
-    grid
     subplot(212)
     plot(states.nerr_log)
     title('num bit errors each frame')
@@ -738,7 +789,7 @@ function rx_bits_log = demod_file(filename, test_frame_mode, noplot=0, EbNodB=10
     clf
     rx_nowave = rx(1000:length(rx));
     subplot(211)
-    plot(rx_nowave(1:states.Fs));
+    plot(real(rx_nowave(1:states.Fs)));
     title('input signal to demod (1 sec)')
     xlabel('Time (samples)');
     axis([1 states.Fs -35000 35000])
@@ -757,21 +808,25 @@ function rx_bits_log = demod_file(filename, test_frame_mode, noplot=0, EbNodB=10
     title('Sample clock (baud rate) offset in PPM');
   end
 
-  if (test_frame_mode == 1) || (test_frame_mode == 6)
+  if (test_frame_mode == 1) || (test_frame_mode == 6) || (test_frame_mode == 8)
     printf("frames: %d Tbits: %d Terrs: %d BER %4.3f EbNo: %3.2f\n", frames, states.Tbits,states. Terrs, states.Terrs/states.Tbits, mean(EbNodB_log));
   end
 
   % we can decode both protocols at the same time
 
-  if (test_frame_mode == 4) || (test_frame_mode == 5)
-    extract_and_print_rtty_packets(states, rx_bits_log, rx_bits_sd_log)
-    corr_log = extract_and_decode_binary_packets(states, rx_bits_log);
-
+  if (test_frame_mode == 4)
+    npackets = extract_and_print_rtty_packets(states, rtty, rx_bits_log, rx_bits_sd_log)
+    printf("Received %d packets\n", npackets);
+  end
+  
+  if (test_frame_mode == 5)
+    corr_log = extract_and_decode_binary_packets(states, binary, rx_bits_log);
+    
     figure(8);
     clf
     plot(corr_log);
     hold on;
-    plot([1 length(corr_log)],[states.binary.uw_thresh states.binary.uw_thresh],'g');
+    plot([1 length(corr_log)],[binary.uw_thresh binary.uw_thresh],'g');
     hold off;
     title('UW correlation');
   end
@@ -782,7 +837,8 @@ endfunction
 % run test functions from here during development
 
 if exist("fsk_horus_as_a_lib") == 0
-  run_sim(1, 2, 100, 9);
+  %run_sim(4, 2, 30, 10);
+  run_sim(5, 4, 30, 100);
   %rx_bits = demod_file("~/Desktop/115.wav",6,0,90);
   %rx_bits = demod_file("fsk_horus.raw",5);
   %rx_bits = demod_file("~/Desktop/4FSK_Binary_NoLock.wav",4);
@@ -795,9 +851,10 @@ if exist("fsk_horus_as_a_lib") == 0
   %rx_bits = demod_file("~/Desktop/fsk_horus_10dB_1000ppm.wav",4);
   %rx_bits = demod_file("~/Desktop/fsk_horus_6dB_0ppm.wav",4);
   %rx_bits = demod_file("test.raw",1,1);
-  %rx_bits = demod_file("/dev/ttyACM0",1);
+  %rx_bits = .rawdemod_file("/dev/ttyACM0",1);
   %rx_bits = demod_file("fsk_horus_rx_1200_96k.raw",1);
   %rx_bits = demod_file("mp.raw",4);
   %rx_bits = demod_file("~/Desktop/launchbox_v2_landing_8KHz_final.wav",4);
   %rx_bits = demod_file("~/Desktop/bench_test_003.wav",7);
+  %rx_bits = demod_file("../build_linux/unittest/fskrx2.raw",8);
 end
