@@ -40,7 +40,7 @@
 % In general, this function processes a bunch of amplitudes, we then
 % use c2sim to hear the results.  Bunch of different experiments below
 
-function [surface mean_f] = newamp1_batch(input_prefix, varargin)
+function [surface_no_mean surface] = newamp1_batch(input_prefix, varargin)
   newamp;
   more off;
 
@@ -56,7 +56,8 @@ function [surface mean_f] = newamp1_batch(input_prefix, varargin)
   vq_search = "mse";
   mode = "const";
   fit_order = 0;
-
+  mean_remove = 1;
+  
   % parse variable argument list
 
   if (length (varargin) > 0)
@@ -71,28 +72,7 @@ function [surface mean_f] = newamp1_batch(input_prefix, varargin)
     if ind
       mode =  varargin{ind+1};
     end
-    ind = arg_exists(varargin, "vq_search");
-    if ind
-      vq_search = varargin{ind+1};
-    end
-    ind = arg_exists(varargin, "vq");
-    if ind
-      vq_type = varargin{ind};
-      vq_filename = varargin{ind+1};
-      printf("vq_type: %s vq_filename: %s vq_search: %s\n", vq_type, vq_filename, vq_search);
-    end
-    ind = arg_exists(varargin, "vqh");
-    if ind
-      vq_type = varargin{ind};
-      vq_filename = varargin{ind+1};
-      printf("vq_type: %s vq_filename: %s vq_search: %s\n", vq_type, vq_filename, vq_search);
-    end
-    ind = arg_exists(varargin, "vql");
-    if ind
-      vq_type = varargin{ind};
-      vq_filename = varargin{ind+1};
-      printf("vq_type: %s vq_filename: %s vq_search: %s\n", vq_type, vq_filename, vq_search);
-    end
+
     ind = arg_exists(varargin, "no_output");
     if ind
       output = 0;
@@ -126,7 +106,18 @@ function [surface mean_f] = newamp1_batch(input_prefix, varargin)
     [model_ surface] = experiment_mel_freq(model, 0, 1, voicing);
   end
   if strcmp(mode, 'const')
-    [model_ surface] = experiment_const_freq(model, vq_type, vq_filename, vq_search);
+    [model_ surface b_log] = experiment_const_freq(model, varargin{:});
+    ind = arg_exists(varargin, "vq_search");
+    if ind
+      if strcmp(varargin{ind+1},"para") || strcmp(varargin{ind+1},"cubic") || strcmp(varargin{ind+1},"sg")
+        fn = sprintf("%s_b_log.txt", output_prefix);
+        save(fn,"b_log");
+      end
+    end
+  end
+  if strcmp(mode, 'pred')
+    [model_ surface b_log] = experiment_const_freq_pred(model, varargin{:});
+    mean_remove = 0;
   end
   if strcmp(mode, 'piecewise')
     model_ = experiment_piecewise(model);
@@ -163,20 +154,20 @@ function [surface mean_f] = newamp1_batch(input_prefix, varargin)
 
         % synthesis phase spectra from magnitiude spectra using minimum phase techniques
 
-        fft_enc = 256;
+        fft_enc = 512;
         phase = determine_phase(model_, f, fft_enc);
         assert(length(phase) == fft_enc);
 
-        % sample phase at centre of each harmonic, not 1st entry Hm[1] in octave Hm[0] in C
+        % sample phase at centre of each harmonic, not 1st entry Hm(1:2) in octave Hm[0] in C
         % is not used
 
         Hm = zeros(1, 2*max_amp);
         for m=1:L
           b = round(m*Wo*fft_enc/(2*pi));
-          Hm(2*m) = cos(phase(b));
-          Hm(2*m+1) = -sin(phase(b));
+          Hm(2*m+1) = cos(phase(b));
+          Hm(2*m+2) = sin(phase(b));
         end
-        fwrite(fhm, Hm, "float32");    
+        fwrite(fhm, Hm, "float32");
       end
     end
  
@@ -198,10 +189,14 @@ function [surface mean_f] = newamp1_batch(input_prefix, varargin)
     end
   end
 
-  for f=1:frames
-    surface(f,:) -= mean(surface(f,:));
+  if mean_remove
+    for f=1:frames
+      surface_no_mean(f,:) = surface(f,:) - mean(surface(f,:));
+    end
+  else
+    surface_no_mean = surface;
   end
-
+  
   printf("\n")
 
 endfunction
@@ -221,7 +216,7 @@ endfunction
 function ind = arg_exists(v, str) 
    ind = 0;
    for i=1:length(v)
-      if strcmp(v{i}, str)
+      if !ind && strcmp(v{i}, str)
         ind = i;
       end     
     end
@@ -231,35 +226,67 @@ endfunction
 % Basic unquantised rate K linear sampling then back to rate L.  Used for generating 
 % training vectors and testing vector quntisers.
 
-function [model_ rate_K_surface] = experiment_const_freq(model, vq_type, vq_filename, vq_search="mse")
+function [model_ rate_K_surface b_log] = experiment_const_freq(model, varargin)
   melvq;
   [frames nc] = size(model);
   Fs = 8000;
   fg = 1;
+  mask_en = 0;
+  vq_search = "gain";   % default to gain search method as it's our favourite atm
+  nvq = 0;              % number of vector quantisers
+  vq_start = [];
   quant_en = 0;
-
+  b_log = [];
+  decimate = 0; decimate_var = 0;
+  
   rate_K_sample_freqs_kHz = [0.1:0.1:4];
   K = length(rate_K_sample_freqs_kHz);
+  
+  % parse command line options
 
-  % optional full band VQ
+  % specify one of more vqs, start index, vq file name, and search method
 
-  if strcmp(vq_type, "vq")
-    quant_en = 1;
-    x = load(vq_filename); vq = x.vq;
-    [vq_rows vq_cols] = size(vq); vq_st = 1; vq_en = vq_cols;
+  ind = anind = arg_exists(varargin, "vq");
+  while anind
+    nvq++;
+    vq_start = [vq_start varargin{ind+1}];
+    avq_filename =  varargin{ind+2};
+    if nvq < 2; vq_filename = avq_filename; else; vq_filename = [vq_filename; avq_filename]; end;
+    avq_search =  varargin{ind+3};
+    if nvq < 2; vq_search = avq_search; else; vq_search = [vq_search; avq_search]; end;
+    printf("nvq %d vq_start: %d vq_filename: %s vq_search: %s\n", nvq, vq_start(nvq), avq_filename, avq_search);
+    anind = arg_exists(varargin(ind+1:length(varargin)), "vq");
+    if anind
+      ind += anind;
+    end
   end
- 
-  if strcmp(vq_type, "vql")
-    quant_en = 1;
-    x = load(vq_filename); vq = x.vq;
-    [vq_rows vq_cols] = size(vq); vq_st = 1; vq_en = vq_st + vq_cols - 1;
+  
+  ind = arg_exists(varargin, "vq_gain");
+  if ind
+    avq_filename = varargin{ind+1};
+    x = load(avq_filename); vq_gain = x.vq;
+    quant_en = 1
   end
 
-  if strcmp(vq_type, "vqh")
-    quant_en = 1;
-    x = load(vq_filename); vq = x.vq;
-    [vq_rows vq_cols] = size(vq); vq_st = 11; vq_en = K;
+  quant_en = arg_exists(varargin, "quant");
+  
+  ind = arg_exists(varargin, "decimate");
+  if ind
+    decimate = varargin{ind+1};
   end
+   
+  % read in optional vector that defines sampling points for
+  % decimation, each entry is the frame number of a sample e.g.
+  % [1 8 10 12 ....
+  
+  ind = arg_exists(varargin, "decsamfile");
+  if ind
+    dec_filename = varargin{ind+1};
+    decvec = load(dec_filename); Ndv = length(decvec);
+    decimate_var = 1;
+  end
+
+  % OK start processing ............................................
 
   energy = zeros(1,frames);
   for f=1:frames
@@ -269,49 +296,224 @@ function [model_ rate_K_surface] = experiment_const_freq(model, vq_type, vq_file
 
   rate_K_surface = resample_const_rate_f(model, rate_K_sample_freqs_kHz, Fs);
 
-  rate_K_surface_fit = zeros(frames,K);
-  b = zeros(1,frames);
+  #{
+  % optional target modification using masking - this didn't help VQ
 
+  if mask_en
+    for f=1:frames
+      rate_K_vec = rate_K_surface(f,:);
+      maskdB = determine_mask(rate_K_vec, rate_K_sample_freqs_kHz, rate_K_sample_freqs_kHz, bark_model=1);
+      target = rate_K_vec;
+      mask_thresh = 6;
+      ind = find (maskdB - target > mask_thresh);
+      target(ind) = maskdB(ind) - mask_thresh;
+      rate_K_surface(f,:) = target;
+    end
+  end  
+  #}
+
+  % remove mean ie global "gain" term
+
+  rate_K_surface_no_mean = zeros(frames,K);
+  meanf = zeros(1,frames);
   for f=1:frames
-    b(f) = mean(rate_K_surface(f,:));
-    rate_K_surface_fit(f,:) = rate_K_surface(f,:) - b(f);
+    meanf(f) = mean(rate_K_surface(f,:));
+    rate_K_surface_no_mean(f,:) = rate_K_surface(f,:) - meanf(f);
   end
  
   % optional vector quantise
   
-  if quant_en
+  if nvq
  
-    rate_K_surface_fit_ = rate_K_surface_fit;
-    res = zeros(frames, vq_cols); ind = [];
+    % note we init with target (ideal) to fill in values not covered by this VQ
 
-    for f=1:frames
-      target = rate_K_surface_fit(f, vq_st:vq_en);
+    rate_K_surface_no_mean_ = rate_K_surface_no_mean;
+    res = zeros(frames, K); ind = zeros(frames, nvq);
 
-      if strcmp(vq_search, "mse")
-        [idx contrib errors test_ g mg sl] = vq_search_mse(vq, target);
-        rate_K_surface_fit_(f, vq_st:vq_en) = contrib;
+    gains = zeros(frames, nvq);
+    
+    % quantise using split VQs
+
+    for i=1:nvq
+      avq_filename = char(cellstr(vq_filename)(i));
+      avq_search = char(cellstr(vq_search)(i));
+      x = load(avq_filename); vq = x.vq; [vq_rows vq_cols] = size(vq);
+      vq_st = vq_start(i); vq_en = vq_st + vq_cols - 1;
+      printf("split VQ: %d vq_filename: %s vq_search: %s vq_st: %d vq_en: %d nVec: %d\n", i, avq_filename, avq_search, vq_st, vq_en, vq_rows);
+
+      if strcmp(avq_search, "mse")
+        [idx contrib errors test_ g mg sl] = vq_search_mse(vq, rate_K_surface_no_mean(:,vq_st:vq_en));
       end
 
-      if strcmp(vq_search, "slope")
-        [idx contrib errors test_ g mg sl] = vq_search_slope(vq, target);
-        rate_K_surface_fit_(f, vq_st:vq_en) = contrib;
+      if strcmp(avq_search, "gain")
+        [idx contrib errors b_log] = vq_search_gain(vq, rate_K_surface_no_mean(:,vq_st:vq_en), weights);
+      end
+
+      if strcmp(avq_search, "max")
+        [idx contrib errors b_log] = vq_search_max(vq, rate_K_surface_no_mean(:,vq_st:vq_en), weights);
+      end
+
+       if strcmp(avq_search, "mg")
+        [idx contrib errors b_log] = vq_search_mg(vq, rate_K_surface_no_mean(:,vq_st:vq_en));
+        gains(:, i) = b_log(:, 2);
+      end
+
+      if strcmp(avq_search, "sg")
+        [idx contrib errors b_log] = vq_search_sg(vq, rate_K_surface_no_mean(:,vq_st:vq_en));
+        b_log = [b_log energy' idx'];
+      end
+
+      if strcmp(avq_search, "slope")
+        [idx contrib errors b_log] = vq_search_slope(vq, rate_K_surface_no_mean(:,vq_st:vq_en),
+                                                     "closed_quant_slope", "open_quant_slope");
+        b_log = [b_log energy' idx'];
+        gains(:, i) = b_log(:, 3);
+      end
+
+      if strcmp(avq_search, "para")
+        [idx contrib errors b_log] = vq_search_para(vq, rate_K_surface_no_mean(:,vq_st:vq_en));
+        b_log = [b_log energy' idx'];
+        
+        if quant_en
+          k = 1:vq_cols; k2 = k.^2;
+#{
+          [nr nc] = size(vq_gain);
+          for f=1:frames
+            v = vq(idx(f),:);
+            b = b_log(f,:);
+            
+            para_target = k2*b(2) + k*b(3) + b(4);
+            
+            % search vq_gain for best match to gain coefficients
+
+            d = g = zeros(nr,1);
+            for r=1:nr
+              g(r) = (sum(para_target) - sum(vq_gain(r,:)))/vq_cols;
+              diff = para_target - (vq_gain(r,:) + g(r));
+              d(r) = diff*diff';
+            end
+            [dmin imin] = min(d);
+            
+            % recalc contrib
+
+            printf("f: %d imin: %d g: %f\n", f, imin, g(imin));
+            contrib(f,:) = b(1)*vq(idx(f),:) + vq_gain(imin,:) + g(imin);
+#}
+            for f=1:frames
+              target = rate_K_surface_no_mean(f,vq_st:vq_en);
+              b = b_log(f,:);
+              para_target = k2*b(2) + k*b(3) + b(4);
+              para_target(1) = quantise([-20 -10 0 10], para_target(1));
+              para_target(10) = quantise([-3 +3], para_target(10));
+              b_ = polyfit([k(1) k(10) k(25)],
+                           [para_target(1) para_target(10) -10],
+                           2);
+              v = vq(idx(f),:);
+              contrib(f,:) = b(1)*v + b_(1)*k2 + b_(2)*k + b_(3);
+              printf("f: %d pt1: %f pt10: %f b_(1): %f b_(2): %f b_(3): %f \n", f, para_target(1), para_target(10), b_(1), b_(2), b_(3));
+            end
+        end
+      end
+
+      if strcmp(avq_search, "cubic")
+        [idx contrib errors b_log] = vq_search_cubic(rate_K_surface_no_mean(:,vq_st:vq_en));
+        b_log = [b_log energy' idx'];
+      end
+
+      rate_K_surface_no_mean_(:, vq_st:vq_en) = contrib; 
+      res(:, vq_st:vq_en) = rate_K_surface_no_mean(:, vq_st:vq_en) - contrib;
+      ind(:,i) = idx;
+    
+      % histograms of higher order gain/shape params
+
+      if strcmp(avq_search, "gain")
+        figure(fg++); clf; hist(b_log, 30); title('gain')
       end
       
-      res(f, vq_st:vq_en) = target - contrib;
-      ind = [ind idx];
+      if strcmp(avq_search, "mg")
+        figure(fg++); clf; hist(b_log(:,1),30); title('mag')
+      end
+      
+      if strcmp(avq_search, "slope")
+
+        hmg = hsl = zeros(1,frames);
+        for f=1:frames
+          hmg(f) = b_log(f, 1);
+          hsl(f) = b_log(f, 2);
+        end
+        figure(fg++); clf; hist(hmg, 30); title('mag')
+        figure(fg++); clf; hist(hsl, 30); title('slope')
+      end
+
+      sd_per_frame = std(res(:,vq_st:vq_en)');
+      t=sprintf("VQ %d", i); 
+      figure(fg++); subplot(211); plot(energy); title(t); subplot(212); plot(sd_per_frame); 
+      figure(fg++); subplot(211); hist(sd_per_frame); title(t); subplot(212); hist(ind(:,i),100);
+      printf("VQ rms SD: %3.2f\n", mean(sd_per_frame));
     end
-    
+ 
     figure(fg++); clf; mesh(res);
-    sd_per_frame = std(res(:,vq_st:vq_en)');
-    figure(fg++); subplot(211); plot(energy); subplot(212); plot(sd_per_frame);
-    figure(fg++); subplot(211); hist(sd_per_frame); subplot(212); hist(ind,100);
-    printf("VQ rms SD: %3.2f\n", mean(sd_per_frame));
+
   else
-    rate_K_surface_fit_ = rate_K_surface_fit;
+    rate_K_surface_no_mean_ = rate_K_surface_no_mean;
   end
 
   for f=1:frames
-    rate_K_surface_(f,:) = rate_K_surface_fit_(f,:) + b(f);
+    rate_K_surface_(f,:) = rate_K_surface_no_mean_(f,:) + meanf(f);
+  end
+
+  % optional decimation
+  
+  if decimate
+    %lf = [1.00 0.75 0.50 0.25];
+    %lf = [1.00 1.00 0.50 0.00];
+    %lf = [1.00 1.00 0.00 0.00];
+    lf = [1.00 0.80 0.20 0.00];
+    
+    for f=1:frames
+
+      % determine frames that bracket the one we need to interp
+
+      left_f = decimate*floor((f-1)/decimate)+1; 
+      right_f = left_f + decimate;
+      if right_f > frames
+        right_f = left_f;
+      end
+
+      % determine fraction of each frame to use
+
+      left_fraction  = lf(mod(f-1,decimate)+1);
+      right_fraction = 1 - left_fraction;
+
+      rate_K_surface_(f,:) = left_fraction*rate_K_surface_(left_f,:) + right_fraction*rate_K_surface_(right_f,:);
+      printf("f: %d left_f: %d right_f: %d left_fraction: %3.2f right_fraction: %3.2f \n", f, left_f, right_f, left_fraction, right_fraction)
+    end
+  end
+  
+  if decimate_var
+    %decvec = 1:4:frames;
+    d = 1; Ndv = length(decvec); right_f = 1;
+    for f=1:frames
+
+      % determine frames that bracket the one we need to interp
+
+      if f == decvec(d)
+        left_f = right_f;
+        if d < Ndv
+          d++;
+          right_f = decvec(d);
+          decimate = decvec(d) - decvec(d-1);
+        end
+      end
+      
+      % determine fraction of each frame to use
+
+      left_fraction  = 1 - (f - left_f)/decimate;
+      right_fraction = 1 - left_fraction;
+
+      rate_K_surface_(f,:) = left_fraction*rate_K_surface_(left_f,:) + right_fraction*rate_K_surface_(right_f,:);
+      printf("f: %d dec: %d lf: %d rf: %d lfrac: %3.2f rfrac: %3.2f \n", f, decimate, left_f, right_f, left_fraction, right_fraction)
+    end
   end
 
   [model_ AmdB_] = resample_rate_L(model, rate_K_surface_, rate_K_sample_freqs_kHz, Fs);
@@ -336,6 +538,215 @@ function [model_ rate_K_surface] = experiment_const_freq(model, vq_type, vq_file
   printf("rate K resampling SD: %3.2f\n", mean(sd));
   figure(fg++); clf; subplot(211); plot(energy); subplot(212); plot(sd); title('sdL');
   figure(fg++); clf; hist(sd);
+
+  figure(fg++); clf; plot(gains);
+  figure(fg++); clf; plot(gains(:,1), gains(:,2), '+');
+endfunction
+
+function model_ = experiment_dct(model)
+
+  [frames tmp] = size(model); max_amp = 160;
+
+  for f=1:frames
+    printf("%d ", f);   
+    Wo = model(f,1);
+    L = min([model(f,2) max_amp-1]);
+    Am = model(f,3:(L+2));
+    AmdB = 20*log10(Am);
+
+    % fit model
+
+    D = dct(AmdB);
+    E = zeros(1,L);
+    E(1:min(20,L)) = D(1:min(20,L));
+    AmdB_ = idct(D);
+
+    model_(f,1) = Wo; model_(f,2) = L; model_(f,3:(L+2)) = 10 .^ (AmdB_(1:L)/20);
+  end
+
+endfunction
+
+
+% Predictive VQ using rate K resampled vectors, and rate 2 decimation
+
+function [model_ rate_K_surface_pred_ b_log] = experiment_const_freq_pred(model, varargin)
+  melvq;
+  [frames nc] = size(model);
+  Fs = 8000;
+  fg = 1;
+  nvq = 0;              % number of vector quantisers
+  vq_start = [];
+  quant_en = 0;
+  b_log = [];
+  decimate = 2;
+  frames2 = floor(frames/decimate);
+  
+  rate_K_sample_freqs_kHz = [0.1:0.1:4];
+  K = length(rate_K_sample_freqs_kHz);
+  
+  % parse command line options
+
+  % set vq search algorithm, e.g. mse, gain, mag, slope.  We've settled on "gain" for now
+  % as slope required extra bits to quantise higher order parameters that offset advantages
+
+  ind = arg_exists(varargin, "vq_search");
+  if ind
+    vq_search = varargin{ind+1};
+  end
+
+  % specify one of more VQs
+
+  ind = anind = arg_exists(varargin, "vq");
+  while anind
+    nvq++;
+    vq_start = [vq_start varargin{ind+1}];
+    avq_filename =  varargin{ind+2};
+    if nvq < 2
+      vq_filename = avq_filename;
+    else
+      vq_filename = [vq_filename; avq_filename];
+    end
+    printf("nvq %d vq_start: %d vq_filename: %s\n", nvq, vq_start(nvq), avq_filename);
+    anind = arg_exists(varargin(ind+1:length(varargin)), "vq");
+    if anind
+      ind += anind;
+    end
+  end
+  
+  ind = arg_exists(varargin, "vq_gain");
+  if ind
+    avq_filename = varargin{ind+1};
+    x = load(avq_filename); vq_gain = x.vq;
+    quant_en = 1
+  end
+
+  ind = arg_exists(varargin, "decimate");
+  if ind
+    decimate = varargin{ind+1};
+  end
+   
+  % OK start processing ............................................
+
+  energy = zeros(1,frames/decimate);
+  for f=1:decimate:frames
+    L = model(f,2);
+    energy(f) = 10*log10(sum( model(f,3:(L+2)) .^ 2 ));
+  end
+
+  rate_K_surface = resample_const_rate_f(model, rate_K_sample_freqs_kHz, Fs);
+
+  % decimate and predict
+  
+  rate_K_surface_pred = zeros(frames2, K);
+  beta = 0.9;
+  g = 1;
+  for f=3:decimate:frames
+    rate_K_surface_pred(g,:) = rate_K_surface(f,:) - beta*rate_K_surface(f-2,:);
+    g++;
+  end
+  
+  rate_K_surface_ = zeros(frames,K);
+  rate_K_surface_(1,:) = rate_K_surface(1,:);
+  g = 1;
+
+  % optional vector quantise
+  
+  if nvq
+ 
+    % note we init with target (ideal) to fill in values not covered by this VQ
+
+    rate_K_surface_pred_ = rate_K_surface_pred;
+    res = zeros(frames2, K); ind = zeros(frames2, nvq);
+
+    % quantise using split VQs
+
+    for i=1:nvq
+      avq_filename = char(cellstr(vq_filename)(i));
+      x = load(avq_filename); vq = x.vq; [vq_rows vq_cols] = size(vq);
+      vq_st = vq_start(i); vq_en = vq_st + vq_cols - 1;
+      printf("split VQ: %d vq_filename: %s vq_st: %d vq_en: %d nVec: %d\n", i, avq_filename, vq_st, vq_en, vq_rows);
+
+      for f=3:decimate:frames
+        rate_K_vec_pred = rate_K_surface(f,:) - beta*rate_K_surface_(f-2,:);
+        
+        if strcmp(vq_search, "mg")
+          [idx contrib errors b_log] = vq_search_mg(vq, rate_K_vec_pred(vq_st:vq_en));
+        end
+        
+        rate_K_vec_pred_ = rate_K_vec_pred;
+        rate_K_vec_pred_(vq_st:vq_en) = contrib; 
+        rate_K_surface_(f,:) = beta*rate_K_surface_(f-2,:) + rate_K_vec_pred_;
+
+        res(:, vq_st:vq_en) = rate_K_surface_pred(:, vq_st:vq_en) - contrib;
+        ind(:,i) = idx;
+      end
+      
+      % histograms of higher order gain/shape params if we are in slope mode
+
+      sd_per_frame = std(res(:,vq_st:vq_en)');
+      t=sprintf("VQ %d", i); 
+      figure(fg++); subplot(211); plot(energy); title(t); subplot(212); plot(sd_per_frame); 
+      figure(fg++); subplot(211); hist(sd_per_frame); title(t); subplot(212); hist(ind(:,i),100);
+      printf("VQ rms SD: %3.2f\n", mean(sd_per_frame));
+    end
+ 
+    figure(fg++); clf; mesh(res);
+
+  else
+    rate_K_surface_pred_ = rate_K_surface_pred;
+    for f=3:decimate:frames
+      rate_K_surface_(f,:) = beta*rate_K_surface_(f-2,:) + rate_K_surface_pred_(g,:);
+      g++;
+    end
+  end
+
+  % re-interpolate back to 10ms rate
+  
+  if decimate
+    for f=1:frames
+
+      % determine frames that bracket the one we need to interp
+
+      left_f = decimate*floor((f-1)/decimate)+1; 
+      right_f = left_f + decimate;
+      if right_f > frames
+        right_f = left_f;
+      end
+
+      % determine fraction of each frame to use
+
+      left_fraction  = 1 - mod((f-1),decimate)/decimate;
+      right_fraction = 1 - left_fraction;
+
+      rate_K_surface_(f,:) = left_fraction*rate_K_surface_(left_f,:) + right_fraction*rate_K_surface_(right_f,:);
+      %printf("f: %d left_f: %d right_f: %d left_fraction: %3.2f right_fraction: %3.2f \n", f, left_f, right_f, left_fraction, right_fraction)
+    end
+  end
+  
+  [model_ AmdB_] = resample_rate_L(model, rate_K_surface_, rate_K_sample_freqs_kHz, Fs);
+
+  % Measure distortion between AmdB and AmdB_, ths includes distortion
+  % in the rate K <-> L transition.  Can optionally plot distorted
+  % frames
+
+  plot_sd_thresh = 5;
+  sd = zeros(1,frames2); g = 1;
+  for f=1:decimate:frames
+    Wo = model(f,1);
+    L = model(f,2);
+    AmdB = 20*log10(model(f,3:(L+2)));
+    sd(g) = std(AmdB(1:L) - AmdB_(f,1:L));
+    if (sd(g) > plot_sd_thresh) && (fg < 10)
+      printf("fg: %d f: %d\n", fg, f);
+      figure(fg++); clf; plot((1:L)*Wo*4/pi, AmdB(1:L),'b+-'); hold on; plot((1:L)*Wo*4/pi, AmdB_(f,1:L),'r+-');
+      plot(rate_K_sample_freqs_kHz, rate_K_surface_(f,:), 'c+-'); hold off;
+    end
+    g++;
+  end
+  printf("rate K resampling SD: %3.2f\n", mean(sd));
+  figure(fg++); clf; subplot(211); plot(energy); subplot(212); plot(sd); title('sdL');
+  figure(fg++); clf; hist(sd);
+   
 endfunction
 
 
@@ -835,7 +1246,7 @@ endfunction
 
 function [model_ rate_K_surface] = experiment_mel_freq(model, vq_en=0, plots=1, voicing)
   [frames nc] = size(model);
-  K = 20; Fs = 16000; correct_rate_K_en = 0;
+  K = 20; Fs = 8000; correct_rate_K_en = 0;
 
   for f=1:frames
     Wo = model(f,1);
@@ -843,7 +1254,7 @@ function [model_ rate_K_surface] = experiment_mel_freq(model, vq_en=0, plots=1,
     Am = model(f,3:(L+2));
     AmdB = 20*log10(Am);
     Am_freqs_kHz = (1:L)*Wo*Fs/(2000*pi);
-    [rate_K_vec rate_K_sample_freqs_kHz] = resample_const_rate_f_mel(model(f,:), K, Fs);
+    [rate_K_vec rate_K_sample_freqs_kHz] = resample_const_rate_f_mel(model(f,:), K, Fs, 'para');
     if correct_rate_K_en
       [tmp_ AmdB_] = resample_rate_L(model(f,:), rate_K_vec, rate_K_sample_freqs_kHz, Fs);
       [rate_K_vec_corrected orig_error error nasty_error_log nasty_error_m_log] = correct_rate_K_vec(rate_K_vec, rate_K_sample_freqs_kHz, AmdB, AmdB_, K, Wo, L, Fs);
@@ -853,7 +1264,11 @@ function [model_ rate_K_surface] = experiment_mel_freq(model, vq_en=0, plots=1,
     end
   end
 
-  model_ = resample_rate_L(model, rate_K_surface, rate_K_sample_freqs_kHz, Fs);
+  if plots
+    mesh(rate_K_surface);
+  end
+  
+  model_ = resample_rate_L(model, rate_K_surface, rate_K_sample_freqs_kHz, Fs, 'para');
  
 endfunction
 
