@@ -35,15 +35,12 @@
 #}
 
 
-% In general, this function processes a bunch of amplitudes, we then
-% use c2sim to hear the results.  Bunch of different experiments below
-
 function tnewamp1(input_prefix)
-  newamp;
+  newamp_700c;
   autotest;
   more off;
 
-  max_amp = 160;
+  max_amp = 80;
   postfilter = 0;   % optional postfiler that runs on Am, not used atm
   synth_phase = 1;
 
@@ -53,7 +50,7 @@ function tnewamp1(input_prefix)
   model_name = strcat(input_prefix,"_model.txt");
   model = load(model_name);
   [frames nc] = size(model);
-  
+
   voicing_name = strcat(input_prefix,"_pitche.txt");
   voicing = zeros(1,frames);
   
@@ -73,6 +70,7 @@ function tnewamp1(input_prefix)
   melvq;
   load train_120_1.txt; load train_120_2.txt;
   train_120_vq(:,:,1)= train_120_1; train_120_vq(:,:,2)= train_120_2; m=5;
+  m=5;
        
   for f=1:frames
     mean_f(f) = mean(rate_K_surface(f,:));
@@ -128,6 +126,15 @@ function tnewamp1(input_prefix)
     voicing_right = voicing(f);
     [Wo_ avoicing_] = interp_Wo_v(Wo_left, Wo_right, voicing_left, voicing_right);
 
+    for i=1:4
+      fprintf(stderr, "  Wo: %4.3f L: %d v: %d\n", Wo_(i), floor(pi/Wo_(i)), avoicing_(i));
+    end
+    fprintf(stderr,"  rate_K_vec: ");
+    for i=1:5
+      fprintf(stderr,"%5.3f  ", rate_K_surface_(f,i));
+    end
+    fprintf(stderr,"\n");
+    
     if f > M
       model_(f-M:f-1,1) = Wo_;
       voicing_(f-M:f-1) = avoicing_;
@@ -151,6 +158,7 @@ function tnewamp1(input_prefix)
           H(k,m) = exp(j*Aw(k, b+1));          
         end     
       end
+
    end
    
    % update for next time
@@ -164,9 +172,9 @@ function tnewamp1(input_prefix)
   figure(1); clf;
   mesh(angle(H));
   figure(2); clf;
-  mesh(angle(H_c));
+  mesh(angle(H_c(:,1:max_amp)));
   figure(3); clf;
-  mesh(abs(H - H_c));
+  mesh(abs(H - H_c(:,1:max_amp)));
 
   check(rate_K_surface, rate_K_surface_c, 'rate_K_surface', 0.01);
   check(mean_f, mean_c, 'mean', 0.01);
@@ -175,7 +183,7 @@ function tnewamp1(input_prefix)
   check(model_(:,1), model__c(:,1), 'interpolated Wo_', 0.001);
   check(voicing_, voicing__c, 'interpolated voicing');
   check(model_(:,3:max_amp+2), model__c(:,3:max_amp+2), 'rate L Am surface ', 0.1);
-  check(H, H_c, 'phase surface');
+  check(H, H_c(:,1:max_amp), 'phase surface');
 
   % Save to disk to check synthesis is OK with c2sim  
 
@@ -216,8 +224,8 @@ function tnewamp1(input_prefix)
 
     Hm = zeros(1, 2*max_amp);
     for m=1:L
-        Hm(2*m+1) = real(H(f,m));
-        Hm(2*m+2) = imag(H(f,m));
+        Hm(2*m) = real(H(f,m));
+        Hm(2*m+1) = -imag(H(f,m));
     end    
     fwrite(fhm, Hm, "float32");    
   end
