@@ -27,6 +27,7 @@
 */
 
 #include "codec2.h"
+#include "c2file.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -41,12 +42,13 @@ int main(int argc, char *argv[])
     FILE          *fout;
     short         *buf;
     unsigned char *bits;
-    int            nsam, nbit, nbyte, gray, softdec;
-    float         *unpacked_bits;
+    int            nsam, nbit, nbyte, gray, softdec, bitperchar;
+    float         *unpacked_bits_float;
+    char          *unpacked_bits_char;
     int            bit, byte,i;
 
     if (argc < 4) {
-	printf("usage: c2enc 3200|2400|1600|1400|1300|1200|700|700B|700C InputRawspeechFile OutputBitFile [--natural] [--softdec]\n");
+	printf("usage: c2enc 3200|2400|1600|1400|1300|1200|700|700B|700C|WB InputRawspeechFile OutputBitFile [--natural] [--softdec] [--bitperchar]\n");
 	printf("e.g    c2enc 1400 ../raw/hts1a.raw hts1a.c2\n");
 	printf("e.g    c2enc 1300 ../raw/hts1a.raw hts1a.c2 --natural\n");
 	exit(1);
@@ -70,8 +72,10 @@ int main(int argc, char *argv[])
 	mode = CODEC2_MODE_700B;
     else if (strcmp(argv[1],"700C") == 0)
 	mode = CODEC2_MODE_700C;
+    else if (strcmp(argv[1],"WB") == 0)
+	mode = CODEC2_MODE_WB;
     else {
-	fprintf(stderr, "Error in mode: %s.  Must be 3200, 2400, 1600, 1400, 1300, 1200, 700, 700B or 700C\n", argv[1]);
+	fprintf(stderr, "Error in mode: %s.  Must be 3200, 2400, 1600, 1400, 1300, 1200, 700, 700B, 700C or WB\n", argv[1]);
 	exit(1);
     }
 
@@ -88,6 +92,21 @@ int main(int argc, char *argv[])
          argv[3], strerror(errno));
 	exit(1);
     }
+    
+    // Write a header if we're writing to a .c2 file
+    char *ext = strrchr(argv[3], '.');
+    if (ext != NULL) {
+        if (strcmp(ext, ".c2") == 0) {
+            struct c2_header out_hdr;
+            memcpy(out_hdr.magic,c2_file_magic,sizeof(c2_file_magic));
+            out_hdr.mode = mode;
+            out_hdr.version_major = CODEC2_VERSION_MAJOR;
+            out_hdr.version_minor = CODEC2_VERSION_MINOR;
+            // TODO: Handle flags (this block needs to be moved down)
+            out_hdr.flags = 0;
+            fwrite(&out_hdr,sizeof(out_hdr),1,fout);
+        };
+    };
 
     codec2 = codec2_create(mode);
     nsam = codec2_samples_per_frame(codec2);
@@ -96,10 +115,10 @@ int main(int argc, char *argv[])
     nbyte = (nbit + 7) / 8;
 
     bits = (unsigned char*)malloc(nbyte*sizeof(char));
-    unpacked_bits = (float*)malloc(nbit*sizeof(float));
+    unpacked_bits_float = (float*)malloc(nbit*sizeof(float));
+    unpacked_bits_char = (char*)malloc(nbit*sizeof(char));
 
-    gray = 1;
-    softdec = 0;
+    gray = 1; softdec = 0; bitperchar = 0;
     for (i=4; i<argc; i++) {
         if (strcmp(argv[i], "--natural") == 0) {
             gray = 0;
@@ -107,6 +126,9 @@ int main(int argc, char *argv[])
         if (strcmp(argv[i], "--softdec") == 0) {
             softdec = 1;
         }
+        if (strcmp(argv[i], "--bitperchar") == 0) {
+            bitperchar = 1;
+        }
     }
     codec2_set_natural_or_gray(codec2, gray);
     //fprintf(stderr,"gray: %d softdec: %d\n", gray, softdec);
@@ -115,19 +137,25 @@ int main(int argc, char *argv[])
 
 	codec2_encode(codec2, bits, buf);
 
-	if (softdec) {
+	if (softdec || bitperchar) {
             /* unpack bits, MSB first, send as soft decision float */
 
             bit = 7; byte = 0;
             for(i=0; i<nbit; i++) {
-                unpacked_bits[i] = 1.0 - 2.0*((bits[byte] >> bit) & 0x1);
+                unpacked_bits_float[i] = 1.0 - 2.0*((bits[byte] >> bit) & 0x1);
+                unpacked_bits_char[i] = (bits[byte] >> bit) & 0x1;
                 bit--;
                 if (bit < 0) {
                     bit = 7;
                     byte++;
                 }
             }
-            fwrite(unpacked_bits, sizeof(float), nbit, fout);
+            if (softdec) {
+                fwrite(unpacked_bits_float, sizeof(float), nbit, fout);
+            }
+            if (bitperchar) {
+                fwrite(unpacked_bits_char, sizeof(char), nbit, fout);
+            }
         }
         else
             fwrite(bits, sizeof(char), nbyte, fout);
@@ -143,7 +171,8 @@ int main(int argc, char *argv[])
 
     free(buf);
     free(bits);
-    free(unpacked_bits);
+    free(unpacked_bits_float);
+    free(unpacked_bits_char);
     fclose(fin);
     fclose(fout);
 
