@@ -48,7 +48,7 @@
 #include "machdep.h"
 #include "bpf.h"
 #include "bpfb.h"
-
+#include "c2wideband.h"
 /*---------------------------------------------------------------------------*\
 
                              FUNCTION HEADERS
@@ -57,7 +57,7 @@
 
 void analyse_one_frame(struct CODEC2 *c2, MODEL *model, short speech[]);
 void synthesise_one_frame(struct CODEC2 *c2, short speech[], MODEL *model,
-			  COMP Aw[]);
+			  COMP Aw[], float gain);
 void codec2_encode_3200(struct CODEC2 *c2, unsigned char * bits, short speech[]);
 void codec2_decode_3200(struct CODEC2 *c2, short speech[], const unsigned char * bits);
 void codec2_encode_2400(struct CODEC2 *c2, unsigned char * bits, short speech[]);
@@ -103,7 +103,7 @@ struct CODEC2 * codec2_create(int mode)
     struct CODEC2 *c2;
     int            i,l;
 
-    if (!((mode >= 0) && (mode <= CODEC2_MODE_700C))) {
+    if (!((mode >= 0) && (mode <= CODEC2_MODE_WB))) {
         return NULL;
     }  
 
@@ -264,6 +264,9 @@ int codec2_bits_per_frame(struct CODEC2 *c2) {
 	return 28;
     if  (c2->mode == CODEC2_MODE_700C)
 	return 28;
+    //TODO: verify this
+    if (c2->mode == CODEC2_MODE_WB)
+	return 64;
 
     return 0; /* shouldn't get here */
 }
@@ -298,7 +301,8 @@ int codec2_samples_per_frame(struct CODEC2 *c2) {
 	return 320;
     if  (c2->mode == CODEC2_MODE_700C)
 	return 320;
-
+    if  (c2->mode == CODEC2_MODE_WB)
+	return 160;
     return 0; /* shouldnt get here */
 }
 
@@ -326,7 +330,10 @@ void codec2_encode(struct CODEC2 *c2, unsigned char *bits, short speech[])
 	codec2_encode_700b(c2, bits, speech);
     if (c2->mode == CODEC2_MODE_700C)
 	codec2_encode_700c(c2, bits, speech);
+    if (c2->mode == CODEC2_MODE_WB)
+	codec2_encode_wb(c2, bits, speech);
 #endif
+
 }
 
 void codec2_decode(struct CODEC2 *c2, short speech[], const unsigned char *bits)
@@ -358,6 +365,8 @@ void codec2_decode_ber(struct CODEC2 *c2, short speech[], const unsigned char *b
  	codec2_decode_700b(c2, speech, bits);
     if (c2->mode == CODEC2_MODE_700C)
  	codec2_decode_700c(c2, speech, bits);
+    if (c2->mode == CODEC2_MODE_WB)
+ 	codec2_decode_wb(c2, speech, bits);
 #endif
 }
 
@@ -496,7 +505,7 @@ void codec2_decode_3200(struct CODEC2 *c2, short speech[], const unsigned char *
 	aks_to_M2(c2->fftr_fwd_cfg, &ak[i][0], LPC_ORD, &model[i], e[i], &snr, 0, 0,
                   c2->lpc_pf, c2->bass_boost, c2->beta, c2->gamma, Aw);
 	apply_lpc_correction(&model[i]);
-	synthesise_one_frame(c2, &speech[c2->n_samp*i], &model[i], Aw);
+	synthesise_one_frame(c2, &speech[c2->n_samp*i], &model[i], Aw, 1.0);
     }
 
     /* update memories for next frame ----------------------------*/
@@ -639,7 +648,7 @@ void codec2_decode_2400(struct CODEC2 *c2, short speech[], const unsigned char *
 	aks_to_M2(c2->fftr_fwd_cfg, &ak[i][0], LPC_ORD, &model[i], e[i], &snr, 0, 0,
                   c2->lpc_pf, c2->bass_boost, c2->beta, c2->gamma, Aw);
 	apply_lpc_correction(&model[i]);
-	synthesise_one_frame(c2, &speech[c2->n_samp*i], &model[i], Aw);
+	synthesise_one_frame(c2, &speech[c2->n_samp*i], &model[i], Aw, 1.0);
     }
 
     /* update memories for next frame ----------------------------*/
@@ -823,7 +832,7 @@ void codec2_decode_1600(struct CODEC2 *c2, short speech[], const unsigned char *
 	aks_to_M2(c2->fftr_fwd_cfg, &ak[i][0], LPC_ORD, &model[i], e[i], &snr, 0, 0,
                   c2->lpc_pf, c2->bass_boost, c2->beta, c2->gamma, Aw);
 	apply_lpc_correction(&model[i]);
-	synthesise_one_frame(c2, &speech[c2->n_samp*i], &model[i], Aw);
+	synthesise_one_frame(c2, &speech[c2->n_samp*i], &model[i], Aw, 1.0);
     }
 
     /* update memories for next frame ----------------------------*/
@@ -993,7 +1002,7 @@ void codec2_decode_1400(struct CODEC2 *c2, short speech[], const unsigned char *
 	aks_to_M2(c2->fftr_fwd_cfg, &ak[i][0], LPC_ORD, &model[i], e[i], &snr, 0, 0,
                   c2->lpc_pf, c2->bass_boost, c2->beta, c2->gamma, Aw);
 	apply_lpc_correction(&model[i]);
-	synthesise_one_frame(c2, &speech[c2->n_samp*i], &model[i], Aw);
+	synthesise_one_frame(c2, &speech[c2->n_samp*i], &model[i], Aw, 1.0);
     }
 
     /* update memories for next frame ----------------------------*/
@@ -1142,6 +1151,7 @@ void codec2_decode_1300(struct CODEC2 *c2, short speech[], const unsigned char *
 
     e_index = unpack_natural_or_gray(bits, &nbit, E_BITS, c2->gray);
     e[3] = decode_energy(e_index, E_BITS);
+    //fprintf(stderr, "%d %f\n", e_index, e[3]);
 
     for(i=0; i<LSP_SCALAR_INDEXES; i++) {
 	lsp_indexes[i] = unpack_natural_or_gray(bits, &nbit, lsp_bits(i), c2->gray);
@@ -1154,7 +1164,7 @@ void codec2_decode_1300(struct CODEC2 *c2, short speech[], const unsigned char *
         model[0].voiced =  model[1].voiced = model[2].voiced = model[3].voiced = 0;
         e[3] = decode_energy(10, E_BITS);
         bw_expand_lsps(&lsps[3][0], LPC_ORD, 200.0, 200.0);
-        fprintf(stderr, "soft mute\n");
+        //fprintf(stderr, "soft mute\n");
     }
 
     /* interpolate ------------------------------------------------*/
@@ -1176,7 +1186,7 @@ void codec2_decode_1300(struct CODEC2 *c2, short speech[], const unsigned char *
 	aks_to_M2(c2->fftr_fwd_cfg, &ak[i][0], LPC_ORD, &model[i], e[i], &snr, 0, 0,
                   c2->lpc_pf, c2->bass_boost, c2->beta, c2->gamma, Aw);
 	apply_lpc_correction(&model[i]);
-	synthesise_one_frame(c2, &speech[c2->n_samp*i], &model[i], Aw);
+	synthesise_one_frame(c2, &speech[c2->n_samp*i], &model[i], Aw, 1.0);
     }
     /*
     for(i=0; i<4; i++) {
@@ -1363,7 +1373,7 @@ void codec2_decode_1200(struct CODEC2 *c2, short speech[], const unsigned char *
 	aks_to_M2(c2->fftr_fwd_cfg, &ak[i][0], LPC_ORD, &model[i], e[i], &snr, 0, 0,
                   c2->lpc_pf, c2->bass_boost, c2->beta, c2->gamma, Aw);
 	apply_lpc_correction(&model[i]);
-	synthesise_one_frame(c2, &speech[c2->n_samp*i], &model[i], Aw);
+	synthesise_one_frame(c2, &speech[c2->n_samp*i], &model[i], Aw, 1.0);
     }
 
     /* update memories for next frame ----------------------------*/
@@ -1561,7 +1571,7 @@ void codec2_decode_700(struct CODEC2 *c2, short speech[], const unsigned char *
 	aks_to_M2(c2->fftr_fwd_cfg, &ak[i][0], LPC_ORD_LOW, &model[i], e[i], &snr, 0, 0,
                   c2->lpc_pf, c2->bass_boost, c2->beta, c2->gamma, Aw);
 	apply_lpc_correction(&model[i]);
-	synthesise_one_frame(c2, &speech[c2->n_samp*i], &model[i], Aw);
+	synthesise_one_frame(c2, &speech[c2->n_samp*i], &model[i], Aw, 1.0);
     }
 
     #ifdef DUMP
@@ -1765,7 +1775,7 @@ void codec2_decode_700b(struct CODEC2 *c2, short speech[], const unsigned char *
 	aks_to_M2(c2->fftr_fwd_cfg, &ak[i][0], LPC_ORD_LOW, &model[i], e[i], &snr, 0, 0,
                   c2->lpc_pf, c2->bass_boost, c2->beta, c2->gamma, Aw);
 	apply_lpc_correction(&model[i]);
-	synthesise_one_frame(c2, &speech[c2->n_samp*i], &model[i], Aw);
+	synthesise_one_frame(c2, &speech[c2->n_samp*i], &model[i], Aw, 1.0);
     }
 
     #ifdef DUMP
@@ -1899,7 +1909,8 @@ void codec2_decode_700c(struct CODEC2 *c2, short speech[], const unsigned char *
 
 
    for(i=0; i<M; i++) {
-	synthesise_one_frame(c2, &speech[c2->n_samp*i], &model[i], &HH[i][0]);
+       /* 700C is a little quiter so lets apply some experimentally derived audio gain */
+       synthesise_one_frame(c2, &speech[c2->n_samp*i], &model[i], &HH[i][0], 1.5);
    }
 }
 
@@ -2025,7 +2036,7 @@ float codec2_get_energy(struct CODEC2 *c2, const unsigned char *bits)
 
 \*---------------------------------------------------------------------------*/
 
-void synthesise_one_frame(struct CODEC2 *c2, short speech[], MODEL *model, COMP Aw[])
+void synthesise_one_frame(struct CODEC2 *c2, short speech[], MODEL *model, COMP Aw[], float gain)
 {
     int     i;
     PROFILE_VAR(phase_start, pf_start, synth_start);
@@ -2055,6 +2066,10 @@ void synthesise_one_frame(struct CODEC2 *c2, short speech[], MODEL *model, COMP
 
     synthesise(c2->n_samp, c2->fftr_inv_cfg, c2->Sn_, model, c2->Pn, 1);
 
+    for(i=0; i<c2->n_samp; i++) {
+        c2->Sn_[i] *= gain;
+    }
+    
     PROFILE_SAMPLE_AND_LOG2(synth_start, "    synth");
 
     ear_protection(c2->Sn_, c2->n_samp);
