@@ -110,6 +110,8 @@ static void fsk_generate_hann_table(struct FSK* fsk){
 }
 #endif
 
+
+
 /*---------------------------------------------------------------------------*\
 
   FUNCTION....: fsk_create_hbr
@@ -150,6 +152,7 @@ struct FSK * fsk_create_hbr(int Fs, int Rs,int P,int M, int tx_f1, int tx_fs)
     fsk->Fs = Fs;
     fsk->Rs = Rs;
     fsk->Ts = Fs/Rs;
+    fsk->burst_mode = 0;
     fsk->N = fsk->Ts*nsyms;
     fsk->P = P;
     fsk->Nsym = nsyms;
@@ -253,6 +256,7 @@ struct FSK * fsk_create_hbr(int Fs, int Rs,int P,int M, int tx_f1, int tx_fs)
     return fsk;
 }
 
+
 #define HORUS_MIN 800
 #define HORUS_MAX 2500
 #define HORUS_MIN_SPACING 100
@@ -298,6 +302,7 @@ struct FSK * fsk_create(int Fs, int Rs,int M, int tx_f1, int tx_fs)
     fsk->Rs = Rs;
     fsk->Ts = Fs/Rs;
     fsk->N = Fs;
+    fsk->burst_mode = 0;
     fsk->P = horus_P;
     fsk->Nsym = fsk->N/fsk->Ts;
     fsk->Ndft = Ndft;
@@ -419,6 +424,17 @@ void fsk_set_nsym(struct FSK *fsk,int nsyms){
     
 }
 
+/* Set the FSK modem into burst demod mode */
+
+void fsk_enable_burst_mode(struct FSK *fsk,int nsyms){
+    fsk_set_nsym(fsk,nsyms);
+    //fsk->nstash = 0;
+    //fsk->Nmem = fsk->N;
+    fsk->nin = fsk->N;
+    fsk->burst_mode = 1;
+    //free(fsk->samp_old);
+    //fsk->samp_old = (COMP*) malloc(1);
+}
 
 void fsk_clear_estimators(struct FSK *fsk){
     int i;
@@ -489,7 +505,6 @@ void fsk_demod_freq_est(struct FSK *fsk, COMP fsk_in[],float *freqs,int M){
     int Fs = fsk->Fs;
     int nin = fsk->nin;
     size_t i,j;
-    int fft_samps;
     float hann;
     float max;
     float tc;
@@ -512,8 +527,6 @@ void fsk_demod_freq_est(struct FSK *fsk, COMP fsk_in[],float *freqs,int M){
     COMP rphi = {.5,0};
     rphi = cmult(cconj(dphi),rphi);
     #endif
-
-    fft_samps = Ndft;
     
     f_min  = (fsk->est_min*Ndft)/Fs;
     f_max  = (fsk->est_max*Ndft)/Fs;
@@ -522,9 +535,18 @@ void fsk_demod_freq_est(struct FSK *fsk, COMP fsk_in[],float *freqs,int M){
     /* scale averaging time constant based on number of samples */
     tc = 0.95*Ndft/Fs;
     
-    int fft_loops = nin/Ndft;
+    int samps;
+    int fft_samps;
+    int fft_loops = nin / Ndft;
+
     for(j=0; j<fft_loops; j++){
-    /* Copy FSK buffer into reals of FFT buffer and apply a hann window */
+        /* 48000 sample rate (for example) will have a spare */
+        /* 896 samples besides the 46 "Ndft" samples, so adjust */
+
+        samps = (nin - ((j + 1) * Ndft));
+        fft_samps = (samps >= Ndft) ? Ndft : samps;
+
+        /* Copy FSK buffer into reals of FFT buffer and apply a hann window */
         for(i=0; i<fft_samps; i++){
             #ifdef USE_HANN_TABLE
             hann = fsk->hann_table[i];
@@ -536,7 +558,8 @@ void fsk_demod_freq_est(struct FSK *fsk, COMP fsk_in[],float *freqs,int M){
             fftin[i].r = hann*fsk_in[i+Ndft*j].real;
             fftin[i].i = hann*fsk_in[i+Ndft*j].imag;
         }
-        /* Zero out the remaining slots */
+
+        /* Zero out the remaining slots on spare samples */
         for(; i<Ndft;i++){
             fftin[i].r = 0;
             fftin[i].i = 0;
@@ -652,6 +675,10 @@ void fsk2_demod(struct FSK *fsk, uint8_t rx_bits[], float rx_sd[], COMP fsk_in[]
     #ifdef MODEMPROBE_ENABLE
     char mp_name_tmp[20]; /* Temporary string for modem probe trace names */
     #endif
+
+    //for(size_t jj = 0; jj<nin; jj++){
+    //    fprintf(stderr,"%f,j%f,",fsk_in[jj].real,fsk_in[jj].imag);
+    //}
     
     /* Load up demod phases from struct */
     for( m=0; m<M; m++)
@@ -693,6 +720,7 @@ void fsk2_demod(struct FSK *fsk, uint8_t rx_bits[], float rx_sd[], COMP fsk_in[]
         /* Back the stored phase off to account for re-integraton of old samples */
         dphi[m] = comp_exp_j(-2*(Nmem-nin-(Ts/P))*M_PI*((fsk->f_est[m])/(float)(Fs)));
         phi_c[m] = cmult(dphi[m],phi_c[m]);
+        //fprintf(stderr,"F%d = %f",m,fsk->f_est[m]);
 
         /* Figure out how much to nudge each sample downmixer for every sample */
         dphi[m] = comp_exp_j(2*M_PI*((fsk->f_est[m])/(float)(Fs)));
@@ -807,6 +835,13 @@ void fsk2_demod(struct FSK *fsk, uint8_t rx_bits[], float rx_sd[], COMP fsk_in[]
         /* Spin the oscillator for the magic line shift */
         phi_ft = cmult(phi_ft,dphift);
     }
+
+    /* Check for NaNs in the fine timing estimate, return if found */
+    /* otherwise segfaults happen */
+    if( isnan(t_c.real) || isnan(t_c.imag)){
+        return;
+    } 
+
     /* Get the magic angle */
     norm_rx_timing =  atan2f(t_c.imag,t_c.real)/(2*M_PI);
     rx_timing = norm_rx_timing*(float)P;
@@ -824,12 +859,15 @@ void fsk2_demod(struct FSK *fsk, uint8_t rx_bits[], float rx_sd[], COMP fsk_in[]
     }
     
     /* Figure out how many samples are needed the next modem cycle */
-    if(norm_rx_timing > 0.25)
-        fsk->nin = N+Ts/2;
-    else if(norm_rx_timing < -0.25)
-        fsk->nin = N-Ts/2;
-    else
-        fsk->nin = N;
+    /* Unless we're in burst mode */
+    if(!fsk->burst_mode){
+        if(norm_rx_timing > 0.25)
+            fsk->nin = N+Ts/2;
+        else if(norm_rx_timing < -0.25)
+            fsk->nin = N-Ts/2;
+        else
+            fsk->nin = N;
+    }
     
     modem_probe_samp_f("t_norm_rx_timing",&(norm_rx_timing),1);
     modem_probe_samp_i("t_nin",&(fsk->nin),1);
@@ -1059,7 +1097,8 @@ void fsk_mod_c(struct FSK *fsk,COMP fsk_out[],uint8_t tx_bits[]){
     int M = fsk->mode;
     COMP dosc_f[M];                 /* phase shift per sample */
     COMP dph;                       /* phase shift of current bit */
-    size_t i,j,m,bit_i,sym;
+    size_t i,j,bit_i,sym;
+    int m;
     
     /* Init the per sample phase shift complex numbers */
     for( m=0; m<M; m++){
