@@ -35,12 +35,13 @@
 #include "comp.h"
 #include "kiss_fft.h"
 
-#define MODEM_STATS_NC_MAX 20
-#define MODEM_STATS_NR_MAX 6
-#define MODEM_STATS_ET_MAX 8
-#define MODEM_STATS_NSPEC            512
-#define MODEM_STATS_MAX_F_HZ         4000
-
+#define MODEM_STATS_NC_MAX    20
+#define MODEM_STATS_NR_MAX    8
+#define MODEM_STATS_ET_MAX    8
+#define MODEM_STATS_NSPEC     512
+#define MODEM_STATS_MAX_F_HZ  4000
+#define MODEM_STATS_MAX_F_EST 4
+      
 struct MODEM_STATS {
     int    Nc;
     float  snr_est;                          /* estimated SNR of rx signal in dB (3 kHz noise BW)  */
@@ -51,13 +52,18 @@ struct MODEM_STATS {
     float  foff;                             /* estimated freq offset in Hz                        */
     float  rx_timing;                        /* estimated optimum timing offset in samples         */
     float  clock_offset;                     /* Estimated tx/rx sample clock offset in ppm         */
-
+    float  sync_metric;                      /* number between 0 and 1 indicating quality of sync  */
+    
     /* eye diagram traces */
     /* Eye diagram plot -- first dim is trace number, second is the trace idx */
     float  rx_eye[MODEM_STATS_ET_MAX][80];
     int    neyetr;                           /* How many eye traces are plotted */
     int    neyesamp;                         /* How many samples in the eye diagram */
 
+    /* optional for FSK modems - est tone freqs */
+
+    float f_est[MODEM_STATS_MAX_F_EST];
+    
     /* Buf for FFT/waterfall */
 
     float        fft_buf[2*MODEM_STATS_NSPEC];
