@@ -12,9 +12,43 @@
 #include <stdio.h>
 #include "mpdecode_core.h"
 
+#define QPSK_CONSTELLATION_SIZE 4
+#define QPSK_BITS_PER_SYMBOL    2
+
+/* QPSK constellation for symbol likelihood calculations */
+
+static COMP S_matrix[] = {
+    { 1.0f,  0.0f},
+    { 0.0f,  1.0f},
+    { 0.0f, -1.0f},
+    {-1.0f,  0.0f}
+};
+         
+
 int extract_output(char out_char[], int DecodedBits[], int ParityCheckCount[], 
                     int max_iter, int CodeLength, int NumberParityBits);
 
+void encode(struct LDPC *ldpc, unsigned char ibits[], unsigned char pbits[]) {
+    unsigned int p, i, tmp, par, prev=0;
+    int          ind;
+    double      *H_rows = ldpc->H_rows;
+
+    for (p=0; p<ldpc->NumberParityBits; p++) {
+        par = 0; 
+
+        for (i=0; i<ldpc->max_row_weight; i++) {
+            ind = (int)H_rows[p + i*ldpc->NumberParityBits];
+            par = par + ibits[ind-1];
+        }
+
+        tmp = par + prev;
+
+        tmp &= 1;    // only retain the lsb 
+        prev = tmp; 
+        pbits[p] = tmp; 
+    }
+}
+
 /* Phi function */
 static float phi0(
 		  float x )
@@ -46,8 +80,7 @@ static float phi0(
   }
 }
 
-static float correction(
-			float xinput )
+static float correction(float xinput )
 {
   if (xinput > 2.625 )
     return( 0 );
@@ -67,6 +100,31 @@ static float LambdaAPPstar(	float mag1,
     return( fabs( mag1 + correction( mag1 + mag2 ) - correction( mag2 - mag1 ) ) );
 }
 
+/* Values for linear approximation (DecoderType=5) */
+
+#define AJIAN -0.24904163195436
+#define TJIAN 2.50681740420944
+
+/* The linear-log-MAP algorithm */
+
+static float max_star0(
+                       float delta1, 
+                       float delta2 )
+{
+    register float diff;
+	
+    diff = delta2 - delta1;
+
+    if ( diff > TJIAN )
+        return( delta2 );
+    else if ( diff < -TJIAN )
+        return( delta1 );
+    else if ( diff > 0 )
+        return( delta2 + AJIAN*(diff-TJIAN) );
+    else
+        return( delta1 - AJIAN*(diff+TJIAN) );
+}
+
 void init_c_v_nodes(struct c_node *c_nodes, 
                     int     shift, 
                     int     NumberParityBits, 
@@ -562,7 +620,7 @@ void SumProduct(	 int	  BitErrors[],
 
 /* Convenience function to call LDPC decoder from C programs */
 
-int run_ldpc_decoder(struct LDPC *ldpc, char out_char[], double input[]) {
+int run_ldpc_decoder(struct LDPC *ldpc, char out_char[], double input[], int *parityCheckCount) {
     int		max_iter, dec_type;
     float       q_scale_factor, r_scale_factor;
     int		max_row_weight, max_col_weight;
@@ -635,6 +693,8 @@ int run_ldpc_decoder(struct LDPC *ldpc, char out_char[], double input[]) {
 
     int iter = extract_output(out_char, DecodedBits, ParityCheckCount, max_iter, CodeLength, NumberParityBits);
 
+    *parityCheckCount = ParityCheckCount[iter-1];
+
     /* Clean up memory */
 
     free(ParityCheckCount);
@@ -702,6 +762,89 @@ void sd_to_llr(double llr[], double sd[], int n) {
 }
 
 
+/*
+   Determine symbol likelihood from received QPSK symbols.
+
+   Notes:
+
+   1) We assume fading[] is real, it is also possible to compute
+      with complex fading, see CML library Demod2D.c source code.
+   2) Using doubles, as experience with FSK in drs232_ldpc.c showed
+      doubles were required to obtain the same answers as Octave.
+*/
+
+void Demod2D(double  symbol_likelihood[],       /* output, M*number_symbols              */
+             COMP    r[],                       /* received QPSK symbols, number_symbols */
+             COMP    S_matrix[],                /* constellation of size M               */
+             float   EsNo,
+             float   fading[],                  /* real fading values, number_symbols    */
+             float   mean_amp,                  
+             int     number_symbols)
+{
+    int     M=QPSK_CONSTELLATION_SIZE;
+    int     i,j;
+    double  tempsr, tempsi, Er, Ei;
+
+    /* determine output */
+  
+    for (i=0;i<number_symbols;i++) {                /* go through each received symbol */
+        for (j=0;j<M;j++) {                         /* each postulated symbol          */
+            tempsr = fading[i]*S_matrix[j].real/mean_amp;
+            tempsi = fading[i]*S_matrix[j].imag/mean_amp;
+            Er = r[i].real/mean_amp - tempsr;
+            Ei = r[i].imag/mean_amp - tempsi;
+            symbol_likelihood[i*M+j] = -EsNo*(Er*Er+Ei*Ei);
+            //printf("symbol_likelihood[%d][%d] = %f\n", i,j,symbol_likelihood[i*M+j]);
+        }
+        //exit(0);
+    }
+
+}
+
+
+void Somap(double  bit_likelihood[],      /* number_bits, bps*number_symbols */
+           double  symbol_likelihood[],   /* M*number_symbols                */
+           int     number_symbols)
+{
+    int    M=QPSK_CONSTELLATION_SIZE, bps = QPSK_BITS_PER_SYMBOL;
+    int    n,i,j,k,mask;
+    double num[bps], den[bps];
+    double metric;
+
+    for (n=0; n<number_symbols; n++) { /* loop over symbols */
+        for (k=0;k<bps;k++) {
+            /* initialize */
+            num[k] = -1000000;			
+            den[k] = -1000000;			
+        }
+ 
+        for (i=0;i<M;i++) {
+            metric =  symbol_likelihood[n*M+i]; /* channel metric for this symbol */
+
+            mask = 1 << (bps - 1);
+            for (j=0;j<bps;j++) {	
+                mask = mask >> 1;
+            }
+            mask = 1 << (bps - 1);
+            
+            for (k=0;k<bps;k++) {	/* loop over bits */
+                if (mask&i) {
+                    /* this bit is a one */
+                    num[k] = max_star0( num[k], metric );
+                } else {
+                    /* this bit is a zero */
+                    den[k] = max_star0( den[k], metric );
+                }
+                mask = mask >> 1;
+            }
+        }
+        for (k=0;k<bps;k++) {
+            bit_likelihood[bps*n+k] = num[k] - den[k];
+        }
+    }
+}
+
+
 int extract_output(char out_char[], int DecodedBits[], int ParityCheckCount[], int max_iter, int CodeLength, int NumberParityBits) {
     int i, j;
 
@@ -713,7 +856,7 @@ int extract_output(char out_char[], int DecodedBits[], int ParityCheckCount[], i
     for (i=0;i<max_iter;i++) {
         if (converged == 0)
             iter++;
-        if ((ParityCheckCount[i] == NumberParityBits)) {
+        if (ParityCheckCount[i] == NumberParityBits) {
             for (j=0; j<CodeLength; j++) {
                 out_char[j] = DecodedBits[i+j*max_iter];
             }
@@ -729,3 +872,15 @@ int extract_output(char out_char[], int DecodedBits[], int ParityCheckCount[], i
     return iter;
 }
 
+void symbols_to_llrs(double llr[], COMP rx_qpsk_symbols[], float rx_amps[], float EsNo, float mean_amp, int nsyms) {
+    int i;
+    
+    double symbol_likelihood[nsyms*QPSK_CONSTELLATION_SIZE];
+    double bit_likelihood[nsyms*QPSK_BITS_PER_SYMBOL];
+
+    Demod2D(symbol_likelihood, rx_qpsk_symbols, S_matrix, EsNo, rx_amps, mean_amp, nsyms);
+    Somap(bit_likelihood, symbol_likelihood, nsyms);
+    for(i=0; i<nsyms*QPSK_BITS_PER_SYMBOL; i++) {
+        llr[i] = -bit_likelihood[i];
+    }
+}
