@@ -376,8 +376,8 @@ void determine_phase(C2CONST *c2const, COMP H[], MODEL *model, int Nfft, codec2_
     float AmdB[MAX_AMP+1], rate_L_sample_freqs_kHz[MAX_AMP+1];
 
     for(m=1; m<=model->L; m++) {
-        AmdB[m] = 20.0*log10f(model->A[m]+1);
-        rate_L_sample_freqs_kHz[m] = (float)m*model->Wo*(c2const->Fs/2000.0)/M_PI;
+        AmdB[m] = 20.0*log10f(model->A[m]);
+        rate_L_sample_freqs_kHz[m] = (float)m*model->Wo*(c2const->Fs/2000.0)/M_PI;        
     }
     
     for(i=0; i<Ns; i++) {
@@ -385,7 +385,6 @@ void determine_phase(C2CONST *c2const, COMP H[], MODEL *model, int Nfft, codec2_
     }
 
     interp_para(Gdbfk, &rate_L_sample_freqs_kHz[1], &AmdB[1], model->L, sample_freqs_kHz, Ns);
-
     mag_to_phase(phase, Gdbfk, Nfft, fwd_cfg, inv_cfg);
 
     for(m=1; m<=model->L; m++) {
