@@ -77,10 +77,12 @@ C2CONST c2const_create(int Fs) {
 
     c2const.tw = Fs*TW_S;
 
+    /*
     fprintf(stderr, "max_amp: %d m_pitch: %d\n", c2const.n_samp, c2const.m_pitch);
     fprintf(stderr, "p_min: %d p_max: %d\n", c2const.p_min, c2const.p_max);
     fprintf(stderr, "Wo_min: %f Wo_max: %f\n", c2const.Wo_min, c2const.Wo_max);
     fprintf(stderr, "nw: %d tw: %d\n", c2const.nw, c2const.tw);
+    */
 
     return c2const;
 }
@@ -326,7 +328,7 @@ void two_stage_pitch_refinement(C2CONST *c2const, MODEL *model, COMP Sw[])
   model->L = floorf(PI/model->Wo);
 
   /* trap occasional round off issues with floorf() */
-  if (model->Wo*model->L >= PI) {
+  if (model->Wo*model->L >= 0.95*PI) {
       model->L--;
   }
   assert(model->Wo*model->L < PI);
